
		// Keep an incrementing menu identifier for all the menus we ever install on any button
		var nextMenuIdentifier = 1;
		
        // We cache the active tab's syndication feeds based on the window's active tab URL
        var windowFeedCache = new Object();

        // We guarantee a "valid" cache entry for the window, so we create it on-the-fly if needed
        function cacheInfoForWindow(theWindow)
        {
			var windowCacheKey;
			
			if (theWindow && theWindow.activeTab)
			{
				windowCacheKey = theWindow.activeTab.url;
			}
			
            if (windowCacheKey == undefined)
            {
                windowCacheKey = "";
            }
            
            var thisCacheInfo = windowFeedCache[windowCacheKey];
            if (thisCacheInfo == undefined)
            {
                windowFeedCache[windowCacheKey] = new Object();
                windowFeedCache[windowCacheKey].haveRequestedFeeds = false;
                windowFeedCache[windowCacheKey].activeTabFeeds = null;
                thisCacheInfo = windowFeedCache[windowCacheKey];
            }
            return thisCacheInfo;
        }                
        
        
        function haveRequestedFeedsForWindow(theWindow)
        {
            var haveRequested = false;
            var thisWindowCache = cacheInfoForWindow(theWindow);
            haveRequested = thisWindowCache.haveRequestedFeeds;
            return haveRequested;
        }
        
        function setHaveRequestedFeedsForWindow(theWindow, haveRequested)
        {
            var thisWindowCache = cacheInfoForWindow(theWindow);
            thisWindowCache.haveRequestedFeeds = haveRequested;
        }


		function setFeedsForWindow(theWindow, theFeedsArray)
		{
            var thisWindowCache = cacheInfoForWindow(theWindow);
			thisWindowCache.activeTabFeeds = theFeedsArray;
		}

		function feedsForWindow(theWindow)
		{
            var thisWindowCache = cacheInfoForWindow(theWindow);
			return thisWindowCache.activeTabFeeds;
		}
        
        function clearCacheInfo()
        {
            windowFeedCache = new Object();
        }

        
        function haveSyndicationFeedForWindow(theWindow)
        {
            var haveFeed = false;
            if (theWindow != undefined)
            {
                if (haveRequestedFeedsForWindow(theWindow) == false)
                {
                    theWindow.activeTab.page.dispatchMessage("globalPageNeedsSyndicationLinks", null);
                    setHaveRequestedFeedsForWindow(theWindow, true);
                }
                
				var windowFeeds = feedsForWindow(theWindow);
				var haveFeed = ((windowFeeds != null) && (windowFeeds.length > 0));
            }
            return haveFeed;
        }
        
		function defaultFeedForWindow(theWindow)
		{
			var defaultFeed = null;
			var currentFeeds = feedsForWindow(theWindow);
			
			if (currentFeeds.length > 0)
			{
				defaultFeed = currentFeeds[0];
			}
			
			return defaultFeed;
		}
        
		function subscribeToFeed(theWindow, theFeed)
		{
			// Convert the URL to a feed:// scheme because Safari
			// will refuse to load e.g. a feed that is listed merely
			// as "text/xml". We do some preflighting of the link rel
			// in the PageLoadEnd.js so we can be more confident it's a
			// good feed: URL.
			var feedURL = theFeed.href;
			if (feedURL.match(/^http:\/\//))
				feedURL = feedURL.replace(/^http:\/\//, "feed://");
			else
				feedURL = "feed:" + feedURL;
				
			theWindow.activeTab.url = feedURL;
		}
		
		safari.application.addEventListener("command", function(event)
        {
            if (event.command == "Subscribe")
            {
                targetWindow = event.target.browserWindow;
                var feedToOpen = defaultFeedForWindow(targetWindow)
                subscribeToFeed(targetWindow, feedToOpen);
            }
        }, false);

		function handleFeedsMenuCommand(event)
		{
			var feedIndex = event.target.identifier;
			var targetWindow = event.currentTarget.browserWindow;
			var currentFeeds = feedsForWindow(targetWindow);
			var selectedFeed = currentFeeds[feedIndex];
			
			subscribeToFeed(targetWindow, selectedFeed);
		}
		
		function updateFeedsMenu(event)
		{
			var menu = event.target;
			var toolbarItem = event.currentTarget;

			var menuItemsLeft = menu.menuItems.length;
			while (menuItemsLeft > 0)
			{
				var thisIndex = menuItemsLeft - 1;
				menu.removeMenuItem(thisIndex);
				menuItemsLeft -= 1;
			}
			
			var feeds = feedsForWindow(toolbarItem.browserWindow);
			for (var i in feeds)
			{
				var thisFeedTitle = feeds[i].title;
				
				// Special case because Blogger includes a redundantly named Atom
				// item for the Comments feed on a blog.
				if ((feeds[i].href.indexOf(".blogspot") != -1) &&
					(feeds[i].href.indexOf("comments/default") != -1))
				{
					thisFeedTitle = thisFeedTitle + " - Comments";
				}
				
				menu.appendMenuItem(i, thisFeedTitle);
			}
		}

        safari.application.addEventListener("validate", function(event)
        {
            var buttonWindow = event.target.browserWindow;
            if (event.command == "Subscribe")
            {
				var myToolbarItem = event.target;
				
                // Only enable the button when we have a syndication feed to open for that window
                var shouldDisable = (haveSyndicationFeedForWindow(buttonWindow) == false);
                myToolbarItem.disabled = shouldDisable;
				
				// Also install a menu if it's not already there
				if ((shouldDisable == false) && (!myToolbarItem.menu))
				{
					var menuIdentifier = nextMenuIdentifier++;
					var feedMenu = safari.extension.createMenu(menuIdentifier);
					myToolbarItem.menu = feedMenu;

					var browserWindow = event.target.browserWindow;
					myToolbarItem.addEventListener("menu", updateFeedsMenu, true);
					myToolbarItem.addEventListener("command", handleFeedsMenuCommand, true);
				}
            }
        }, false);
        
         function revalidateButtons()
         {
            for (thisButtonIndex = 0; thisButtonIndex < safari.extension.toolbarItems.length; thisButtonIndex++)                   
            {
                var thisButton = safari.extension.toolbarItems[thisButtonIndex];
                thisButton.validate();
            }
        }
        
        // We pay attention to activate, and other events so we can update our cached 
        // knowledge of the page's Atom or RSS feed links

        function forceRevalidation()
        {
            clearCacheInfo();
                        
            revalidateButtons();
        }
                 
        safari.application.addEventListener("navigate", function(event)
        {
            forceRevalidation();
        }, true);
        
        // We inject PageLoadStart.js into every page where it scans for the 
        // presence of an Atom or RSS feed, and caches the URLs for us.
        safari.application.addEventListener("message", function(event)
        {        
            if (event.name == "currentSyndicationLinks")
            {
                var theWindow = event.target.browserWindow;
				
				setFeedsForWindow(theWindow, event.message);
                
                revalidateButtons();
            }
        }, true);